#include "stdafx.h"
#include "..\..\DuiLib\UIlib.h"
#include "ChatRoomList.hpp"
#include "main_frame.hpp"
#include "ChatDlg.h"
#include "NeMessengerDlg.h"
#include <algorithm>

namespace DuiLib
{

ChatRoomItem::ChatRoomItem()
{
	pNtalk = NULL;
	pChatDlg = NULL;
	bGroupChat = FALSE;
	eMsgGB = CHAT_MSG_NONE;
	eStatusType = ustOFFLINE;
    nRoomID = -1;
}

ChatRoomItem::ChatRoomItem(const NE_MYROOM_DATA& myRoomData)
{
	pNtalk = NULL;
	pChatDlg = NULL;
	nRoomID = myRoomData.nRoomID;
	eMsgGB = CHAT_MSG_NONE;
	eStatusType = ustOFFLINE;
	if (myRoomData.msgType == DEF_CHAT_MSG_TYPE_STICKER)
	{
		eMsgGB = CHAT_MSG_STICKER;
		strLastMsg = GetLastFileOrStickerMsg(myRoomData.strLastMsgUserId.c_str());
	}
	else if (myRoomData.msgType == DEF_FTP_MSG_TYPE)
	{
		eMsgGB = CHAT_MSG_FTP;
		strLastMsg = GetLastFileOrStickerMsg(myRoomData.strLastMsgUserId.c_str());
	}
	else if (myRoomData.msgType == DEF_CHAT_MSG_TYPE_CHAT)
	{
		eMsgGB = CHAT_MSG_CHAT;
		strLastMsg = myRoomData.strLastMsg.c_str();
	}
	CTime tmLastMsg = myRoomData.nLastMsgTime;
	GetLocaleTime(&tmLastMsg);
	tmLastTime = tmLastMsg;
	bGroupChat = (myRoomData.nRoomType == CHATROOM_MULTITYPE);

	size_t chatUsrCnt =  myRoomData.listChatMemberData.size();
	for (size_t i=0; i<chatUsrCnt; ++i)
	{
		if (myRoomData.listChatMemberData[i].strID != g_pMessengerActor->GetUserID())
		{
			strName += myRoomData.listChatMemberData[i].strName.c_str();
			strName += SPLIT_STRING;
			strUserID += myRoomData.listChatMemberData[i].strID.c_str();
			strUserID += SPLIT_STRING;
		}
	}
	if (chatUsrCnt == 1)
	{
		strName = LoadStringFromResource(IDS_STRING_NO_CONTACT);
		strUserID = _T("");
	}
	else
	{
		strName = strName.Left(strName.GetLength() - 2);
		strUserID = strUserID.Left(strUserID.GetLength() - 2);
	}
	
	if (bGroupChat)
	{
		strImage =GetAppDUIImgDir() + _T("\\img_main_chat_list_profile_nopic.png");
	} 
	else
	{
		strImage = GET_APP()->GetProfileImagePath(strUserID);
	}
}

ChatRoomItem::ChatRoomItem(CChatDlg*  pChat)
{
	CopyChatItem(pChat);
}

void ChatRoomItem::CopyChatItem(CChatDlg*  pChat)
{
	if (pChat != NULL)
	{
		pNtalk = NULL;
		pChatDlg = pChat;
		nRoomID = pChat->GetRoomID();
		strName = pChat->GetAllUserNameFromList();
		strUserID = pChat->GetAllUserIDFromList();
		tmLastTime = pChat->GetCreationTime();
		bGroupChat = pChat->GetChatModel()->IsGroupRoom();
		CChatItem* pChatItem = pChat->GetLastMsgItem();
		pChat->SetLastFileOrStickerMsg(pChatItem);
		if (pChatItem != NULL)
		{
			eMsgGB = pChatItem->GetItemType();
			if (pChatItem->GetChatItemInfo()->eMsgGB == CHAT_MSG_CHAT || pChatItem->GetChatItemInfo()->eMsgGB == CHAT_MSG_SYS)
			{
				strLastMsg = pChatItem->GetChatItemInfo()->strMsg;
			}
			else if ( pChatItem->GetChatItemInfo()->eMsgGB == CHAT_MSG_STICKER || pChatItem->GetChatItemInfo()->eMsgGB == CHAT_MSG_FTP)
			{
				strLastMsg = pChatDlg->GetLastFileOrStickerMsg();
			}
		}
		else
		{
			eMsgGB = CHAT_MSG_CHAT;
			strLastMsg = _T("");
		}

		if (bGroupChat)
		{
			strImage = GetAppDUIImgDir() + _T("\\img_main_chat_list_profile_nopic.png");
		}
		else
		{
			eStatusType = ustOFFLINE;
			strImage = GET_APP()->GetProfileImagePath(strUserID);
		}
	}
}

BOOL ChatRoomItem::IsInChatItem(CString strKeyword)
{
	if ((eMsgGB == CHAT_MSG_CHAT) && (strLastMsg.Find(strKeyword) != -1))
	{
		return TRUE;
	}
	return FALSE;
}

size_t ChatRoomItem::GetOnlineUserCount()
{
	size_t nOnlineUserCount = 0;
	CString strID = strUserID;
	if (strID.GetLength() < 1)
	{
		nOnlineUserCount = 0;
	}
	if (strID.Find(SPLIT_STRING) != -1)
	{
		std::vector<CString> vecUserID;
		size_t nUserIDCount = Split(strID, SPLIT_STRING, false, vecUserID);

		for (size_t i=0; i<vecUserID.size(); ++i)
		{
			const IGTBuddyUser* pUser = g_pMessengerActor->GetBuddyInfoFromPersist(vecUserID[i]);
			if ((pUser != NULL) && (ustOFFLINE != pUser->GetStatus()))
			{
				++nOnlineUserCount;
			}
		}
	}
	else
	{
		const IGTBuddyUser* pUser = g_pMessengerActor->GetBuddyInfoFromPersist(strID);
		if ((pUser != NULL) && (ustOFFLINE != pUser->GetStatus()))
		{
			++nOnlineUserCount;
		}
	}

	return nOnlineUserCount;
}

size_t ChatRoomItem::GetTotalUserCount()
{
	size_t nTotalUserCount = 0;
	CString strID = strUserID;
	if (strID.GetLength() < 1)
	{
		nTotalUserCount = 0;
	}
	if (strID.Find(SPLIT_STRING) != -1)
	{
		std::vector<CString> vecUserID;
		size_t nUserIDCount = Split(strID, SPLIT_STRING, false, vecUserID);

		nTotalUserCount = nUserIDCount + 1;
	}
	else
	{
		nTotalUserCount = 2;
	}

	return nTotalUserCount;
}

CString ChatRoomItem::GetLastFileOrStickerMsg(CString strLastUserID)
{
	CString strMyID = g_pMessengerActor->GetUserID();
	CString strFileOrStickerMsg = _T("");
	if (eMsgGB == CHAT_MSG_FTP )
	{
		if (!strMyID.Compare(strLastUserID))
		{
			strFileOrStickerMsg = LoadStringFromResource(IDS_STRING_SEND_FILE);
		}
		else{
			strFileOrStickerMsg = LoadStringFromResource(IDS_STRING_RECEIVE_FILE);
		}
	}
	else if (eMsgGB == CHAT_MSG_STICKER)
	{
		if (!strMyID.Compare(strLastUserID))
		{
			strFileOrStickerMsg = LoadStringFromResource(IDS_STRING_SEND_STICKER);
		}
		else{
			strFileOrStickerMsg = LoadStringFromResource(IDS_STRING_RECEIVE_STICKER);
		}

	}
	return strFileOrStickerMsg;
}

void ChatRoomItem::UpdataChatItem()
{
	const IGTBuddyUser* pUser = g_pMessengerActor->GetBuddyInfoFromPersist(strUserID);
	if (pUser != NULL && !bGroupChat)
	{
		eStatusType = pUser->GetStatus();
		const NE_MEMBER_DATA* pMemberData = g_pMessengerActor->GetEmployeeInfo(strUserID);
		if (pMemberData != NULL)
		{
			strEMail = pMemberData->strEmail.c_str();
		} 

	}
}

BOOL ChatRoomItem::RequestChatLeaveInRecentChatList(CString strSrvAddress, int nPort, int nRoomID, CString strMyID, CString strMyName, CString strMyTalkName)
{
	pNtalk = CreateNTalkClient(this);
	ASSERT(pNtalk != NULL);
	if (pNtalk->CreateChatSession(strSrvAddress, nPort, nRoomID, strMyID, strMyName, strMyTalkName, FALSE) == FALSE)
	{
		delete pNtalk;
		pNtalk = NULL;
		return FALSE;		
	}
	return TRUE;
}

void CALLBACK ChatRoomItem::NtalkClient_OnCreateChatSession(INTalkClient* pNTClient, NTalkCreateChatType chatType, NTalkCreateSesionErrorCode eErrorCode)
{
	// Create session successfully, then requestChatLeave and delete this
	if (pNtalk)
	{
		pNtalk->RequestChatLeave();
		pNtalk->DestroyChatSession();				
		delete pNtalk;
		pNtalk = NULL;
	}
	delete this;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CChatRoomList::CChatRoomList(CPaintManagerUI& paint_manager)
	: root_node_(NULL)
	, delay_deltaY_(0)
	, delay_number_(0)
	, delay_left_(0)
	, level_text_start_pos_(0)
	, text_padding_(0, 0, 0, 0)
	, paint_manager_(paint_manager)
{
	m_nItemNormalWidth = 47;
	m_nItemNormalHeight = 47;
	m_nItemSelectedWidth = 47;
	m_nItemSelectedHeight = 47;
	SetItemShowHtml(true);

	root_node_ = new Node;
	root_node_->data().level_ = -1;
	root_node_->data().child_visible_ = true;
	root_node_->data().has_child_ = true;
	root_node_->data().list_elment_ = NULL;
	m_bSingleSelect = false;
	m_bSingleControlSelect = false;
	
}

CChatRoomList::~CChatRoomList()
{
	if (root_node_)
		delete root_node_;

	root_node_ = NULL;
}

bool CChatRoomList::Add(CControlUI* pControl)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	return CListUI::Add(pControl);
}

bool CChatRoomList::AddAt(CControlUI* pControl, int iIndex)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	return CListUI::AddAt(pControl, iIndex);
}

bool CChatRoomList::Remove(CControlUI* pControl)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	if (reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()) == NULL)
		return CListUI::Remove(pControl);
	else
		return RemoveNode(reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()));
}

bool CChatRoomList::RemoveAt(int iIndex)
{
	CControlUI* pControl = GetItemAt(iIndex);
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	if (reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()) == NULL)
		return CListUI::RemoveAt(iIndex);
	else
		return RemoveNode(reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()));
}

void CChatRoomList::RemoveAll()
{
	CListUI::RemoveAll();
	int nCount = root_node_->num_children();
	for (int i = nCount - 1; i >=0;  i--)
	{
		Node* child = root_node_->child(i);
		RemoveNode(child);
	}
	delete root_node_;

	root_node_ = new Node;
	root_node_->data().level_ = -1;
	root_node_->data().child_visible_ = true;
	root_node_->data().has_child_ = true;
	root_node_->data().list_elment_ = NULL;
}

void CChatRoomList::DoEvent(TEventUI& event) 
{
	if (!IsMouseEnabled() && event.Type > UIEVENT__MOUSEBEGIN && event.Type < UIEVENT__MOUSEEND)
	{
		if (m_pParent != NULL)
			m_pParent->DoEvent(event);
		else
			CVerticalLayoutUI::DoEvent(event);
		return;
	}

	if (event.Type == UIEVENT_TIMER && event.wParam == SCROLL_TIMERID)
	{
		if (delay_left_ > 0)
		{
			--delay_left_;
			SIZE sz = GetScrollPos();
			LONG lDeltaY = 0;
            if(delay_number_ > 0)
            {
                lDeltaY = (LONG)(CalculateDelay((double)delay_left_ / delay_number_) * delay_deltaY_);
            }
			if ((lDeltaY > 0 && sz.cy != 0)  || (lDeltaY < 0 && sz.cy != GetScrollRange().cy ))
			{
				sz.cy -= lDeltaY;
				SetScrollPos(sz);
				return;
			}
		}
		delay_deltaY_ = 0;
		delay_number_ = 0;
		delay_left_ = 0;
		m_pManager->KillTimer(this, SCROLL_TIMERID);
		return;
	}
	if (event.Type == UIEVENT_SCROLLWHEEL)
	{
		LONG lDeltaY = 0;
		if (delay_number_ > 0)
			lDeltaY =  (LONG)(CalculateDelay((double)delay_left_ / delay_number_) * delay_deltaY_);
		switch (LOWORD(event.wParam))
		{
		case SB_LINEUP:
			if (delay_deltaY_ >= 0)
				delay_deltaY_ = lDeltaY + 8;
			else
				delay_deltaY_ = lDeltaY + 12;
			break;
		case SB_LINEDOWN:
			if (delay_deltaY_ <= 0)
				delay_deltaY_ = lDeltaY - 8;
			else
				delay_deltaY_ = lDeltaY - 12;
			break;
		}
		if
			(delay_deltaY_ > 100) delay_deltaY_ = 100;
		else if
			(delay_deltaY_ < -100) delay_deltaY_ = -100;

		delay_number_ = (DWORD)sqrt((double)abs(delay_deltaY_)) * 5;
		delay_left_ = delay_number_;
		m_pManager->SetTimer(this, SCROLL_TIMERID, 50U);
		return;
	}
	if(event.Type == UIEVENT_RBUTTONUP)
	{
		m_pManager->SendNotify(this, DUI_MSGTYPE_ITEMCLICK, UIEVENT_RBUTTONUP, -1);
	}

	CListUI::DoEvent(event);
}

Node* CChatRoomList::GetRoot()
{
	return root_node_;
}

const TCHAR* const kLogoButtonControlName = _T("logo");
const TCHAR* const kLogoContainerControlName = _T("logo_container");
const TCHAR* const kNickNameControlName = _T("nickname");
const TCHAR* const kArrowIconControlName = _T("arrowicon");
const TCHAR* const kDescriptionControlName = _T("description");
const TCHAR* const kOperatorPannelControlName = _T("operation");

static bool OnLogoButtonEvent(void* event) 
{
	TEventUI *pEvent = (TEventUI*)event;

    if( ((TEventUI*)event)->Type == UIEVENT_BUTTONDOWN ) 
	{
		CControlUI* pButton = pEvent->pSender;
		if( pButton != NULL ) 
		{
			CString name = pButton->GetName();
			if ( 0 == _tcsicmp(pButton->GetName(),_T("iptbtn")))
			{
				if(GET_APP()->IsAvailableIPT())
				{
					CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
					if (pListElement)
					{
                       Node* pNode = (Node*)pListElement->GetTag();
					   if (pNode != NULL)
					   {
						   NTIPTStatusType iptMyStatus = g_pMessengerActor->GetMyIptStatus();
						   NTIPTStatusType iptStatusOfBuddy =GET_MESSENGER_DLG()->GetMainFrame()->GetMemberIPTStatus(pNode->data().value);
						 //  const NE_MEMBER_DATA MembereData = GET_MESSENGER_DLG()->GetWorkGroupController()->m_pModel->GetMember(pNode->data().value);
						   const NE_MEMBER_DATA MembereData = pNode->member_data();
						   CMenu menuList;
						   menuList.LoadMenu(IDR_MEMU_COMMON_MENU);
						   CMenu* pPopup = menuList.GetSubMenu(7);
						   GET_MESSENGER_DLG()->GetMainFrame()->MakeIPTMenu(iptMyStatus, iptStatusOfBuddy, &MembereData, pPopup);

						   if( pPopup != NULL )
						   {
							   CPoint point(pEvent->ptMouse.x, pEvent->ptMouse.y);
							   ::ClientToScreen(GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd(), &point);
							   int nCmdID = pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RETURNCMD | TPM_RIGHTBUTTON, point.x, point.y, CWnd::FromHandle(GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd()));
							   GET_MESSENGER_DLG()->GetMainFrame()->OnMenuIPT(&MembereData, nCmdID);
						   }
					   }
					}
			
				}
			} 
			else
			{	
				CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
				if( pListElement != NULL ) 
				{
					CChatRoomList *pThis = (CChatRoomList *)pListElement->GetOwner();
					CButtonUI* logo_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
					if (logo_button!=NULL)
					{
						CRect rcItem = pButton->GetPos();

						Node* pNode = (Node*)pListElement->GetTag();
						if (pNode != NULL && !pNode->folder())
						{
							RECT rcClient = {0};
							GetWindowRect(GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd(), &rcClient);
							RECT rcAround = {rcClient.left + rcItem.left - 6, rcClient.top + rcItem.top, rcClient.left + rcItem.right, rcClient.top + rcItem.bottom};
							if (GET_CONFIG()->EnableFunction(_T("UI_03")))
							{
								GET_MESSENGER_DLG()->GetProfileDlgController()->CreateProfileDlg(pNode->data().value.GetData(), rcAround, GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd());
							}
						}
					}
				}
			}	
		}
    }
	else if( ((TEventUI*)event)->Type == UIEVENT_MOUSEENTER  || ((TEventUI*)event)->Type == UIEVENT_MOUSELEAVE )
	{
		CControlUI* pButton = pEvent->pSender;
		if( pButton != NULL ) 
		{
			CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
            if( pListElement != NULL )
			{
				CChatRoomList *pThis = (CChatRoomList *)pListElement->GetOwner();

				if (1)
				{
					CLabelUI* profile = static_cast<CLabelUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("profile")));
					Node* pNode = (Node*)pListElement->GetTag();
					if((profile != NULL) && (pNode != NULL))
					{
						if (!pNode->folder())
						{
							TCHAR szBuf[MAX_PATH] = {};
							_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), _T("skin\\nTalkRes\\img_main_chat_list_profile_bar.png"));
							if(((TEventUI*)event)->Type == UIEVENT_MOUSEENTER)
							{
								profile->SetBkImage(szBuf);
							}
							else
							{
								profile->SetBkImage(_T(""));
							}
						}
						
					}

					CButtonUI* logo = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("logo")));
					if(logo != NULL)
					{
						TCHAR szBuf[MAX_PATH] = {};
						if(((TEventUI*)event)->Type == UIEVENT_MOUSEENTER)
						{
							_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), _T("skin\\nTalkRes\\img_main_chat_list_profile_sel.png"));
						}
						else
						{
							_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), _T("skin\\nTalkRes\\img_main_chat_list_profile_nor.png"));
						}
						logo->SetForeImage(szBuf);
					}
				}
			}
		}
	}

    return true;
}

Node* CChatRoomList::GetNodeByIndex(int index){
  ASSERT(root_node_);
  return root_node_->child(index);

}

Node* CChatRoomList::FindNodeByCompareData(int nCpr)
{
	for (int i = 0; i < root_node_->num_children(); i++)
	{
		if(root_node_->child(i)->GetCompareData() == nCpr)
		{
			return root_node_->child(i);
		}
	}
		
	return NULL;
}

Node* CChatRoomList::AddChattingRoomNode(const ChatRoomItem& item, Node* parent)
{
	if (!parent)
		parent = root_node_;

	CString strBuf = _T("");

	CListContainerElementUI* pListElement = NULL;
	if( !m_dlgBuilder.GetMarkup()->IsValid() ) 
	{
		pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create(_T("chatRoom_list_item.xml"), (UINT)0, NULL, &paint_manager_));
	}
	else 
	{
		pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create((UINT)0, &paint_manager_));
	}
	if (pListElement == NULL)
		return NULL;

	Node* node = new Node;
	node->SetComapreData(item.nRoomID);
	node->data().level_ = parent->data().level_ + 1;
	node->data().has_child_ = false;
	node->data().folder_ = item.bGroupChat;
	node->data().child_visible_ = (node->data().level_ == 0);
	node->data().child_visible_ = false;
	node->data().text_ = item.strName;
	node->data().value = item.strUserID;
	node->data().list_elment_ = pListElement;

	if (!parent->data().child_visible_)
		pListElement->SetVisible(false);

	if (parent != root_node_ && !parent->data().list_elment_->IsVisible())
		pListElement->SetVisible(false);

	CDuiRect rcPadding = text_padding_;
	for (int i = 0; i < node->data().level_; ++i)
	{
		rcPadding.left += level_text_start_pos_;		
	}
	pListElement->SetPadding(rcPadding);

	CButtonUI* logo_button = static_cast<CButtonUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
	if (logo_button != NULL)
	{
		strBuf = item.strImage;
		logo_button->SetBkImage(strBuf);
		logo_button->SetTag((UINT_PTR)pListElement);
		logo_button->OnEvent += MakeDelegate(&OnLogoButtonEvent);
		if (item.bGroupChat)
		{
			logo_button->SetToolTip(LoadStringFromResource(IDS_STRING_RECENT_CHAT_TOOLTIP_1));
		}
		else
		{
			strBuf.Format(LoadStringFromResource(IDS_STRING_RECENT_CHAT_TOOLTIP_3), item.strName.GetData(), 
				GetPresenceTip(item.eStatusType),item.strEMail.GetData());
			logo_button->SetToolTip(strBuf);
		}
	}

	CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kNickNameControlName));
	if (nick_name != NULL)
	{
		strBuf = item.strName;
		nick_name->SetText(strBuf);
		nick_name->SetTextColor(0xFF000000);
		if (item.bGroupChat)
		{
			nick_name->SetToolTip(item.strName);
		}
		else
		{
			strBuf.Format(LoadStringFromResource(IDS_STRING_RECENT_CHAT_TOOLTIP_3), item.strName.GetData(), 
				GetPresenceTip(item.eStatusType),item.strEMail.GetData());
			nick_name->SetToolTip(strBuf);
		}
	}

	CLabelUI* lastmsg = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, _T("lastmsg")));
	if (lastmsg != NULL)
	{
		strBuf = item.strLastMsg;
		strBuf.Replace(_T("\r\n"), _T(""));
		lastmsg->SetText(strBuf);
		lastmsg->SetToolTip(item.strLastMsg);
	}
	CLabelUI* date = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, _T("date")));
	if(date != NULL)
	{
		date->SetText(ParseChatTime(item.tmLastTime));
	}
	CLabelUI* pLabProfile = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, _T("profile")));
	if (pLabProfile != NULL)
	{
		pLabProfile->SetTag((UINT_PTR)item.bGroupChat);
	}

	pListElement->SetFixedHeight(58);
	pListElement->SetTag((UINT_PTR)node);

	if (!CListUI::AddAt(pListElement, 0))
	{
		delete pListElement;
		delete node;
		node = NULL;
	}

	parent->insert_child(node, 0);
	return node;
}

bool CChatRoomList::RemoveNode(Node* node)
{
	if (!node || node == root_node_) return false;


	int nCount = node->num_children();
	for (int i = nCount - 1; i >=0;  i--)
	{
		Node* child = node->child(i);
		RemoveNode(child);
	}

	CListUI::Remove(node->data().list_elment_);
	node->parent()->remove_child(node);
	delete node;
	node = NULL;

	return true;
}

void CChatRoomList::SetNodeEditable(Node* node, bool bEdit)
{
	if (!node || node == root_node_)
		return;

	CEditUI* edit_groupname = static_cast<CEditUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, _T("edit_groupname")));
	CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kNickNameControlName));

	if(nick_name != NULL && edit_groupname != NULL)
	{
		CDuiString strNickName  = node->data().text_;
		CDuiString strEditText = edit_groupname->GetText();

		if(!bEdit)
		{
			CDuiString oldName;
            //nick_name->SetText(strEditText);
			IGTBuddyGroup* pGroup = (IGTBuddyGroup*)node->GetNodeUserData();
            if (pGroup)
            {
                oldName = pGroup->GetName();
                if(oldName != strEditText)
                {
                    GET_MESSENGER_DLG()->ProcRenameGroupName(pGroup->GetID(), strEditText.GetData());
                }
                edit_groupname->SetVisible(false);
            }
		}
		else
		{
			IGTBuddyGroup* pGroup = (IGTBuddyGroup*)node->GetNodeUserData();
            if (pGroup)
            {
                edit_groupname->SetText(pGroup->GetName());
                edit_groupname->SetTag((UINT_PTR)node);
                edit_groupname->SetVisible(true);
                edit_groupname->SetFocus();
                edit_groupname->SetSelAll();
            }
		}

		nick_name->SetVisible(!bEdit);

	}
}

void CChatRoomList::SetChildVisible(Node* node, bool visible)
{
	CDuiString html_text = _T("");
	if (!node || node == root_node_)
		return;

	if(node->data().child_visible_ == visible)
	{
		return;
	}

	node->data().child_visible_ = visible;
	
	TCHAR szBuf[MAX_PATH] = {0};
	
	if (node->data().has_child_)
	{
#if defined(UNDER_WINCE)
		_stprintf(szBuf, _T("<x %d>"), 10);
#else
		_stprintf_s(szBuf, MAX_PATH - 1, _T("<x %d>"), 1);
#endif
		html_text += szBuf;
		html_text += node->data().text_;

		CLabelUI* arrow_icon = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kArrowIconControlName));
		if (arrow_icon != NULL)
		{
			if (node->data().child_visible_)
			{
				if(!node->data().selected_)
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowup_nor.png"));
				}
				else
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowup_sel.png"));
				}
			}
			else
			{
				if(!node->data().selected_)
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowdown_nor.png"));
				}
				else
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowdown_sel.png"));
				}
			}
		}

		CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kNickNameControlName));
		if (nick_name != NULL)
		{
			nick_name->SetShowHtml(true);
			nick_name->SetText(html_text);
			nick_name->NeedParentUpdate();
		}
	}

	/*if (!node->data().list_elment_->IsVisible())
		return;*/

	if (!node->has_children())
		return;

	Node* begin = node->child(0);
	Node* end = node->get_last_child();
	int nDelayMs = 0;
    int step = 1;
	int ChildCnt = end->data().list_elment_->GetIndex() - begin->data().list_elment_->GetIndex()  +1;
	
	if (ChildCnt <= 5 && ChildCnt > 0){
		nDelayMs =  150/ChildCnt;
		step = 1;
	}
	else if (ChildCnt <  20 && ChildCnt > 5)
	{
		nDelayMs =  100/ChildCnt;
		step = 1;
	}
	else 
	{
		nDelayMs =  15; 
		step = ChildCnt/20 + 1;
	}
		

    if (visible == false)
    {
		for (int i = end->data().list_elment_->GetIndex(); i >= begin->data().list_elment_->GetIndex(); --i)
		{
			if (nDelayMs > 0)
			{
				if ((i+1 - begin->data().list_elment_->GetIndex())%step == 0)
				{
				/*	CString str;
					str.Format(_T("sleep:%d\n"),i+1 - begin->data().list_elment_->GetIndex());
					OutputDebugString(str);*/
					Sleep(nDelayMs);

				}

			} 

			CControlUI* control = NULL;	

			control = GetItemAt(i);		

			if (_tcsicmp(control->GetClass(), _T("ListContainerElementUI")) == 0)
			{
				if (visible) 
				{

					Node* local_parent = ((Node*)control->GetTag())->parent();
					if (local_parent->data().child_visible_)
					{
						control->SetVisible(true);
						if (nDelayMs > 0)
						{
							if ((i+1 - begin->data().list_elment_->GetIndex())%step == 0)
							{
								control->Invalidate(TRUE);

							}

						} 
					}
				}
				else
				{
					Node* local_parent = ((Node*)control->GetTag())->parent();

					control->SetVisible(false);	
					if (nDelayMs > 0)
					{
						if ((i+1 - begin->data().list_elment_->GetIndex())%step == 0)
						{
							UpdateWindow(GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd());

						}

					} 

				}
			}
		}
    } 
    else
    {
		for (int i = begin->data().list_elment_->GetIndex(); i <= end->data().list_elment_->GetIndex(); ++i)
		{
			if (nDelayMs > 0)
			{
				if ((i+1 - begin->data().list_elment_->GetIndex())%step == 0)
				{
					/*CString str;
					str.Format(_T("sleep:%d\n"),i+1 - begin->data().list_elment_->GetIndex());
					OutputDebugString(str);*/
					Sleep(nDelayMs);

				}

			} 

			CControlUI* control = NULL;	

			control = GetItemAt(i);		

			if (_tcsicmp(control->GetClass(), _T("ListContainerElementUI")) == 0)
			{
				if (visible) 
				{

					Node* local_parent = ((Node*)control->GetTag())->parent();
					if (local_parent->data().child_visible_)
					{
						control->SetVisible(true);
						if (nDelayMs > 0)
						{
							if ((i+1 - begin->data().list_elment_->GetIndex())%step == 0)
							{
								control->Invalidate(TRUE);

							}

						} 
					}
				}
				else
				{
					Node* local_parent = ((Node*)control->GetTag())->parent();

					control->SetVisible(false);	
					if (nDelayMs > 0)
					{
						if ((i+1 - begin->data().list_elment_->GetIndex())%step == 0)
						{
							UpdateWindow(GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd());

						}

					} 

				}
			}
		}
    }
	
}

bool CChatRoomList::CanExpand(Node* node) const
{
	if (!node || node == root_node_)
		return false;

	return node->data().has_child_;
}

list<Node*> CChatRoomList::GetSelectedItems()
{
	list<Node*> listRet;
	list<int> listSelectedIndexs = CListUI::GetSelectedItems();
	for(list<int>::iterator it = listSelectedIndexs.begin();
		it != listSelectedIndexs.end(); it++)
	{
		CControlUI* pControl = GetItemAt(*it);
		//Node* node = NULL;
		if( pControl != NULL) 
		{
			Node* node = (Node*)pControl->GetTag();
			if(node != NULL)
			{
				listRet.push_back(node);
			}
		}
	}
	
	return listRet;
}

list<Node*> CChatRoomList::GetAllItems()
{
	list<Node*> listRet;

	listRet.clear();

	for(int i = 0; i < GetCount(); i++)
	{
		CControlUI* pControl = GetItemAt(i);

		Node* node = NULL;

		if( pControl != NULL) 
		{
			node = (Node*)pControl->GetTag();

			if(node != NULL)
			{
				listRet.push_back(node);
			}
		}
	}

	return listRet;
}

void CChatRoomList::SetSelectItemSize(int nWidth, int nHeight)
{
	m_nItemSelectedWidth = nWidth;
	m_nItemSelectedHeight = nHeight;
}

void CChatRoomList::SetNormalItemSize(int nWidth, int nHeight)
{
	m_nItemNormalWidth = nWidth;
	m_nItemNormalHeight = nHeight;
}

void CChatRoomList::SetSelectViewModal( CControlUI* pControl, bool bSelect)
{
	CListContainerElementUI* pFriendListItem = static_cast<CListContainerElementUI*>(pControl);
	Node* node = (Node*)pControl->GetTag();

	node->data().selected_ = bSelect;
	if (node->data().has_child_)
	{
		CLabelUI* arrow_icon = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kArrowIconControlName));
		if (arrow_icon != NULL)
		{
			if (node->data().child_visible_)
			{
				if(!node->data().selected_)
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowup_nor.png"));
				}
				else
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowup_sel.png"));
				}
			}
			else
			{
				if(!node->data().selected_)
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowdown_nor.png"));
				}
				else
				{
					arrow_icon->SetBkImage(_T("img_main_favorite_arrowdown_sel.png"));
				}
			}
		}
	}
	SetChildVisible(node,node->data().child_visible_);

	if ((pFriendListItem != NULL) && (node != NULL) && !node->folder())
	{
		CContainerUI* pOperatorPannel = static_cast<CContainerUI*>(m_pManager->FindSubControlByName(pFriendListItem, kOperatorPannelControlName));
		if (pOperatorPannel != NULL)
		{
			pOperatorPannel->SetVisible(bSelect);
		}
	}
}

CString CChatRoomList::ParseChatTime(CTime tmChat)
{
	CString strChatTime = tmChat.FormatGmt(_T("%Y.%m.%d"));;
	CTime tmNow = CTime::GetCurrentTime();
	GetLocaleTime(&tmNow);
	CString strNowTime = tmNow.FormatGmt(_T("%Y.%m.%d"));
	CTimeSpan tmSpan= tmNow - tmChat;
	LONGLONG nSecond = tmSpan.GetTotalSeconds();

	if (strChatTime == strNowTime)
	{
		if (nSecond >= 0 && nSecond < 60)
		{
			nSecond = max(nSecond, 1);
			strChatTime.Format(_T("%d"), (INT)nSecond);
			strChatTime += LoadStringFromResource(IDS_STRING_SECOND_AGO);
		}
		else if (nSecond >= 60 && nSecond < 3600)
		{
			strChatTime.Format(_T("%d"), (INT)(nSecond/60));
			strChatTime += LoadStringFromResource(IDS_STRING_MINUTE_AGO); 
		}
		else if (nSecond >= 3600 && nSecond < 3600*24)
		{
			strChatTime = tmChat.FormatGmt(_T("%H:%M"));
		}
	}
	else
	{
		strChatTime = tmChat.FormatGmt(_T("%Y.%m.%d"));
	}

	return strChatTime;
}

} // namespace DuiLib